/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.system.sharedarea;

import java.util.Collection;

import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.jbi.servicedesc.endpoint.Location;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint;
import org.ow2.petals.jbi.servicedesc.endpoint.PetalsServiceEndpoint.EndpointType;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.implementation.MutableImplementation;
import org.ow2.petals.microkernel.api.jbi.messaging.endpoint.exception.EndpointAlreadyExistsException;
import org.ow2.petals.microkernel.api.server.FractalHelper;
import org.ow2.petals.microkernel.api.system.sharedarea.exception.SharedAreaException;
import org.w3c.dom.Document;

/**
 * Interface of the component responsible of the shared area management. The shared area is used a distributed shared
 * memory to store information shared by all containers of a same domain.
 * 
 * It is responsible of actually adding the container to its current topology in the shared memory on component start.
 * But in order to leave a topology, one must call {@link #leaveTopology()}: when the component is stopped, the
 * container is not removed from the topology in the shared memory.
 * 
 * The idea is that adding or removing a container to a topology is a administration operation.
 * 
 * All the other operations are related to endpoint management: the topology operations are reduced to their minimum in
 * this interface.
 * 
 * @author ddesjardins - EBM WebSourcing
 * @author Olivier Fabre - EBM WebSourcing
 * @author Christophe DENEUX - Linagora
 * @author Victor Noel - Linagora
 */
public interface SharedAreaService extends MutableImplementation {

    public static final String COMPONENT_LOGGER_NAME = "Petals.System.SharedAreaService";

    /**
     * Name of the server interface of the Fractal component {@value FractalHelper#SHARED_AREA_COMPONENT}
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * The name of the client interface of the component {@value FractalHelper#SHARED_AREA_COMPONENT} for the
     * configuration service.
     */
    public final static String CONFIGURATION_FRACTAL_ITF_NAME = "configuration";

    /**
     * <p>
     * Register an internal endpoint in the shared area.
     * </p>
     *
     * <b>Note</b>: If the endpoint already exists and <code>canOverwrite</code> is set to <code>true</code>, it is
     * overridden, else, the method returns <code>false</code>.
     * 
     * @param endpointName
     *            Endpoint name
     * @param serviceName
     *            Endpoint service name.
     * @param interfaces
     *            Endpoint interfaces
     * @param location
     *            Endpoint location. Not <code>null</code>.
     * @param description
     *            Endpoint WSDL
     * @param canOverwrite
     *            If set to <code>true</code>, if the endpoint already exists, it will be overwritten, otherwise the
     *            error {@link EndpointAlreadyExistsException} is thrown.
     * @return A {@link PetalsServiceEndpoint} for the registered endpoint or <code>null</code> if the endpoint already
     *         exists, and <code>canOverwrite</code> is not set to <code>true</code>.
     * @throws SharedAreaException
     *             An error occurs registering the endpoint
     */
    public PetalsServiceEndpoint registerInternalEndpoint(String endpointName, QName serviceName, QName[] interfaces,
            Location location, Document description, boolean canOverwrite) throws SharedAreaException;

    /**
     * <p>
     * Register an endpoint in the shared area.
     * </p>
     *
     * <b>Note</b>: If the endpoint already exists and <code>canOverwrite</code> is set to <code>true</code>, it is
     * overridden, else, the method returns <code>false</code>.
     * 
     * @param endpoint
     *            Endpoint
     * @param location
     *            Endpoint location. Not <code>null</code>.
     * @param description
     *            Endpoint WSDL to register
     * @param canOverwrite
     *            If set to <code>true</code>, if the endpoint already exists, it will be overwritten.
     * @return A {@link PetalsServiceEndpoint} for the registered endpoint or <code>null</code> if the endpoint already
     *         exists, and <code>canOverwrite</code> is not set to <code>true</code>.
     * @throws SharedAreaException
     *             An error occurs registering the endpoint
     */
    public PetalsServiceEndpoint registerExternalEndpoint(ServiceEndpoint endpoint, Location location,
            Document description, boolean canOverwrite) throws SharedAreaException;

    /**
     * Deregister the given endpoint.
     * 
     * @param serviceName
     *            qualified name of the endpoint's service; must be non null
     * @param endpointName
     *            name of the endpoint; must be non null
     * @return the removed endpoint
     * @throws SharedAreaException
     *             An error occurred deregistering the endpoint
     */
    public PetalsServiceEndpoint deregisterEndpoint(QName serviceName, String endpointName)
            throws SharedAreaException;

    /**
     * Check if an endpoint is registered
     * 
     * @param serviceName
     *            qualified name of the endpoint's service; must be non null
     * @param endpointName
     *            name of the endpoint; must be non null
     * @return <code>true</code> if the endpoint is registered
     * @throws SharedAreaException
     *             if an error occurs checking the endpoint existence
     */
    public boolean isRegisteredEndpoint(QName serviceName, String endpointName) throws SharedAreaException;

    /**
     * <p>
     * Retrieve registered endpoints. The returned collection is not backed by the shared area.
     * </p>
     * 
     * @param type
     *            The type that the endpoints must implement. Can be <code>null</code>.
     * @return The endpoints matching '<code>type</code>' or all endpoints if '<code>type</code>' is <code>null</code>
     */
    public Collection<PetalsServiceEndpoint> getEndpoints(EndpointType type) throws SharedAreaException;

    /**
     * <p>
     * Get the service endpoint for the named registered endpoint, if any.
     * </p>
     * 
     * <p>
     * Note: this method could be implemented at endpoint directory level getting all endpoints and filtering them, but
     * for performance reasons some optimization can be done if the shared area provides directly such a method.
     * </p>
     * 
     * @param serviceName
     *            qualified name of the endpoint's service; must be non null
     * @param endpointName
     *            name of the endpoint; must be non null
     * @return the named endpoint, or null if the named endpoint is not registered
     */
    public PetalsServiceEndpoint getEndpoint(QName serviceName, String endpointName) throws SharedAreaException;

    /**
     * <p>
     * Get endpoints implementing the interface '<code>interfaceName</code>'. The returned collection is not backed by
     * the shared area.
     * </p>
     * 
     * @param interfaceName
     *            qualified name of interface implemented by the endpoints; must be non-null
     * @param type
     *            The type that the endpoints must implement. Can be <code>null</code>.
     * @return the registered endpoints for the specified interface name;
     * @throws SharedAreaException
     *             A parameter is invalid, or an error occurs searching endpoints
     */
    public Collection<PetalsServiceEndpoint> getEndpointsForInterface(QName interfaceName, EndpointType type)
            throws SharedAreaException;

    /**
     * <p>
     * Get endpoints implementing the service '<code>serviceName</code>'. The returned collection is not backed by the
     * shared area.
     * </p>
     * 
     * @param serviceName
     *            The service that the endpoints must implement. Must be non-null.
     * @param type
     *            The type that the endpoints must implement. Can be <code>null</code>.
     * @return The endpoints matching '<code>serviceName</code>'
     * @throws SharedAreaException
     *             A parameter is invalid, or an error occurs searching endpoints
     */
    public Collection<PetalsServiceEndpoint> getEndpointsForService(QName serviceName, EndpointType type)
            throws SharedAreaException;

    /**
     * Returns the ServiceEndpoint description as a DOM Document
     * 
     * @param endpoint
     *            the endpoint
     * @return a DOM representation of a ServiceEndpoint description or <code>null</code> if there is no such
     *         description
     * @throws SharedAreaException
     *             if an error occurs during endpoint description retrieval
     */
    public Document getDescription(ServiceEndpoint endpoint) throws SharedAreaException;

    /**
     * Returns an endpoint description as a DOM Document
     * 
     * @param serviceName
     *            the service name
     * @param endpointName
     *            the endpoint name
     * @return a DOM representation of a ServiceEndpoint description or <code>null</code> if there is no such
     *         description
     * @throws SharedAreaException
     *             if an error occurs during endpoint description retrieval
     */
    public Document getDescription(QName serviceName, String endpointName) throws SharedAreaException;

    /**
     * Leave the current topology (if it's a dynamic topology, we remove ourselves from it) and stops listening to
     * topology changes.
     * 
     * TODO should we do something specific such a throwing errors if the shared area is used after we left the
     * topology?!
     * 
     * It also removes all the local endpoints. The returned collection is not backed by the shared area.
     * 
     * @return the list of removed endpoints
     * @throws SharedAreaException
     */
    public Collection<SavedEndpoint> leaveTopology() throws SharedAreaException;

    /**
     * Join the current topology configured in the {@link ConfigurationService} (if it's a dynamic topology, we add
     * ourselves to it) and start listening to topology changes.
     * 
     * It also activate all the local endpoints passed as argument.
     * 
     * @param endpoints
     *            the list of previously removed endpoints
     * @throws SharedAreaException
     */
    public void joinTopology(Collection<SavedEndpoint> endpoints) throws SharedAreaException;

    /**
     * Load the topology configured in the {@link ConfigurationService} or from the shared area and start listening to
     * topology changes.
     * 
     * It also activate all the local endpoints passed as argument.
     * 
     * @param endpoints
     *            the list of previously removed endpoints
     * @throws SharedAreaException
     */
    public void loadTopology(Collection<SavedEndpoint> endpoints) throws SharedAreaException;

    public class SavedEndpoint {

        public final Document description;

        public final PetalsServiceEndpoint endpoint;

        public SavedEndpoint(final PetalsServiceEndpoint endpoint, final Document description) {
            this.endpoint = endpoint;
            this.description = description;
        }
    }
}
